@@ -56,22 +56,31 @@ except RuntimeError:
     pass
 
 import getopt
-import gnome
 import os
 import signal
 import time
 import unicodedata
 
+import settings
+if settings.useDBus:
+    import dbus.mainloop.glib
+    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
+    import dbusserver
+    try:
+        import gsmag as mag
+    except:
+        import mag
+else:
+    import mag
+
 import pyatspi
 import braille
 import debug
 import httpserver
 import keynames
 import keybindings
-import mag
 import orca_state
 import platform
-import settings
 import speech
 
 from input_event import BrailleEvent
@@ -79,9 +88,6 @@ from input_event import KeyboardEvent
 from input_event import MouseButtonEvent
 from input_event import keyEventToString
 
-if settings.useDBus:
-    import dbusserver
-
 from orca_i18n import _           # for gettext support
 
 if settings.debugMemoryUsage:
@@ -100,6 +106,15 @@ _userSettings = None
 #
 _commandLineSettings = {}
 
+# True if --debug is used on the command line.
+#
+_debugSwitch = False
+
+# Filename of the debug file to use if --debug or --debug-file is
+# used on the command line.
+#
+_debugFile = None
+
 ########################################################################
 #                                                                      #
 # METHODS FOR HANDLING PRESENTATION MANAGERS                           #
@@ -176,7 +191,7 @@ def getScriptForApp(app):
 #                                                                      #
 ########################################################################
 
-def setLocusOfFocus(event, obj, notifyPresentationManager=True):
+def setLocusOfFocus(event, obj, notifyPresentationManager=True, force=False):
     """Sets the locus of focus (i.e., the object with visual focus) and
     notifies the current presentation manager of the change.
 
@@ -184,9 +199,11 @@ def setLocusOfFocus(event, obj, notifyPresentationManager=True):
     - event: if not None, the Event that caused this to happen
     - obj: the Accessible with the new locus of focus.
     - notifyPresentationManager: if True, propagate this event
+    - force: if True, don't worry if this is the same object as the
+      current locusOfFocus
     """
 
-    if obj == orca_state.locusOfFocus:
+    if not force and obj == orca_state.locusOfFocus:
         return
 
     # If this event is not for the currently active script, then just return.
@@ -488,6 +505,22 @@ def isNavigationKey(event_string):
                   "orca.isNavigationKey: returning: %s" % reply)
     return reply
 
+def isDiacriticalKey(event_string):
+    """Return an indication of whether this is a non-spacing diacritical key
+
+    Arguments:
+    - event: the event string
+    - modifiers: key modifiers state
+
+    Returns True if this is a non-spacing diacritical key
+    """
+
+    reply = event_string.startswith("dead_")
+
+    debug.println(debug.LEVEL_FINEST,
+                  "orca.isDiacriticalKey: returning: %s" % reply)
+    return reply
+
 class KeyEventType:
     """Definition of available key event types."""
 
@@ -515,6 +548,9 @@ class KeyEventType:
     # A navigation key event.
     NAVIGATION = 'NAVIGATION'
 
+    # A diacritical key event.
+    DIACRITICAL = 'DIACRITICAL'
+
     def __init__(self):
         pass
 
@@ -535,7 +571,7 @@ def keyEcho(event):
     #
     if orca_state.locusOfFocus \
         and (orca_state.locusOfFocus.getRole() == pyatspi.ROLE_PASSWORD_TEXT):
-        return
+        return False
 
     event_string = event.event_string
     debug.println(debug.LEVEL_FINEST,
@@ -549,31 +585,42 @@ def keyEcho(event):
 
         if isModifierKey(event_string):
             if not settings.enableModifierKeys:
-                return
+                return False
             eventType = KeyEventType.MODIFIER
 
         elif isNavigationKey(event_string):
             if not settings.enableNavigationKeys:
-                return
+                return False
             eventType = KeyEventType.NAVIGATION
 
+        elif isDiacriticalKey(event_string):
+            if not settings.enableDiacriticalKeys:
+                return False
+            eventType = KeyEventType.DIACRITICAL
+
         elif isPrintableKey(event_string):
-            if not settings.enablePrintableKeys:
-                return
+            if not (settings.enablePrintableKeys \
+                    or settings.enableEchoByCharacter):
+                return False
             eventType = KeyEventType.PRINTABLE
 
         elif isLockingKey(event_string):
             if not settings.enableLockingKeys:
-                return
+                return False
             eventType = KeyEventType.LOCKING
 
             modifiers = event.modifiers
 
+            brailleMessage = None
             if event_string == "Caps_Lock":
                 if modifiers & (1 << pyatspi.MODIFIER_SHIFTLOCK):
                     eventType = KeyEventType.LOCKING_UNLOCKED
+                    brailleMessage = keynames.getKeyName(event_string) \
+                                     + " " + _("off")
                 else:
                     eventType = KeyEventType.LOCKING_LOCKED
+                    brailleMessage = keynames.getKeyName(event_string) \
+                                     + " " + _("on")
 
             elif event_string == "Num_Lock":
                 # [[[TODO: richb - we are not getting a correct modifier
@@ -587,30 +634,55 @@ def keyEcho(event):
                 #    eventType = KeyEventType.LOCKING_LOCKED
                 pass
 
+            if brailleMessage:
+                braille.displayMessage(brailleMessage,
+                                       flashTime=settings.brailleFlashTime)
+
         elif isFunctionKey(event_string):
             if not settings.enableFunctionKeys:
-                return
+                return False
             eventType = KeyEventType.FUNCTION
 
         elif isActionKey(event_string):
             if not settings.enableActionKeys:
-                return
+                return False
             eventType = KeyEventType.ACTION
 
         else:
             debug.println(debug.LEVEL_FINEST,
                   "orca.keyEcho: event string not handled: %s" % event_string)
-            return
-
-        debug.println(debug.LEVEL_FINEST,
-                      "orca.keyEcho: speaking: %s" % event_string)
+            return False
 
         # We keep track of the time as means to let others know that
         # we are probably echoing a key and should not be interrupted.
         #
         orca_state.lastKeyEchoTime = time.time()
 
-        speech.speakKeyEvent(event_string, eventType)
+        # Before we echo printable keys, let's try to make sure the
+        # character echo isn't going to also echo something.
+        #
+        characterEcho = \
+            eventType == KeyEventType.PRINTABLE \
+            and not _orcaModifierPressed \
+            and orca_state.activeScript \
+            and orca_state.activeScript.willEchoCharacter(event)
+
+        # One last check for echoing -- a PRINTABLE key may have squeaked
+        # through due to the enableEchoByCharacter check above.  We only
+        # want to echo PRINTABLE keys if enablePrintableKeys is True.
+        #
+        if not (characterEcho or
+                (eventType == KeyEventType.PRINTABLE \
+                 and not settings.enablePrintableKeys)):
+            debug.println(debug.LEVEL_FINEST,
+                          "orca.keyEcho: speaking: %s" % event_string)
+            speech.speakKeyEvent(event_string, eventType)
+            return True
+        elif characterEcho:
+            debug.println(debug.LEVEL_FINEST,
+                          "orca.keyEcho: letting character echo handle: %s" \
+                          % event_string)
+            return False
 
 def _setClickCount(inputEvent):
     """Sets the count of the number of clicks a user has made to one
@@ -686,6 +758,19 @@ def _processKeyboardEvent(event):
     """
     global _orcaModifierPressed
 
+    # Input methods appear to play games with repeating events
+    # and also making up events with no timestamps.  We try
+    # to handle that here. See bug #589504.
+    #
+    if (event.timestamp == 0) \
+       or (event.timestamp == orca_state.lastInputEventTimestamp \
+           and orca_state.lastInputEvent \
+           and orca_state.lastInputEvent.hw_code == event.hw_code \
+           and orca_state.lastInputEvent.type == event.type):
+        debug.println(debug.LEVEL_FINE, keyEventToString(event))
+        debug.println(debug.LEVEL_FINE, "IGNORING EVENT DUE TO TIMESTAMP")
+        return
+
     orca_state.lastInputEventTimestamp = event.timestamp
 
     # Log the keyboard event for future playback, if desired.
@@ -699,6 +784,9 @@ def _processKeyboardEvent(event):
 
     keyboardEvent = KeyboardEvent(event)
 
+    if keyboardEvent.type == pyatspi.KEY_PRESSED_EVENT:
+        braille.killFlash()
+
     # See if this is one of our special Orca modifier keys.
     #
     # Just looking at the keycode should suffice, but there is a
@@ -748,7 +836,10 @@ def _processKeyboardEvent(event):
         #
         if not settings.learnModeEnabled and \
            orca_state.activeScript.echoKey(keyboardEvent):
-            keyEcho(keyboardEvent)
+            try:
+                keyEcho(keyboardEvent)
+            except:
+                debug.printException(debug.LEVEL_SEVERE)
 
     elif isOrcaModifier \
         and (keyboardEvent.type == pyatspi.KEY_RELEASED_EVENT):
@@ -821,11 +912,11 @@ def _processKeyboardEvent(event):
 #                                                                      #
 ########################################################################
 
-def _processBrailleEvent(command):
+def _processBrailleEvent(event):
     """Called whenever a  key is pressed on the Braille display.
 
     Arguments:
-    - command: the BrlAPI command for the key that was pressed.
+    - command: the BrlAPI event for the key that was pressed.
 
     Returns True if the event was consumed; otherwise False
     """
@@ -834,9 +925,9 @@ def _processBrailleEvent(command):
 
     # Braille key presses always interrupt speech.
     #
-    speech.stop()
-
-    event = BrailleEvent(command)
+    event = BrailleEvent(event)
+    if event.event['command'] not in braille.dontInteruptSpeechKeys:
+        speech.stop()
     orca_state.lastInputEvent = event
 
     try:
@@ -916,6 +1007,10 @@ def loadUserSettings(script=None, inputEvent=None):
     else:
         try:
             _userSettings = __import__("user-settings")
+            if _debugSwitch:
+                debug.debugLevel = debug.LEVEL_ALL
+                debug.eventDebugLevel = debug.LEVEL_OFF
+                debug.debugFile = open(_debugFile, 'w', 0)
         except ImportError:
             debug.printException(debug.LEVEL_FINEST)
         except:
@@ -1223,7 +1318,7 @@ def start(registry):
     if settings.cacheValues:
         pyatspi.setCacheLevel(pyatspi.CACHE_PROPERTIES)
 
-    registry.start()
+    registry.start(gil=settings.useGILIdleHandler)
 
 def die(exitCode=1):
 
@@ -1361,6 +1456,24 @@ def usage():
     print "-l, --list-apps              " + \
           _("Print the known running applications")
 
+    # Translators: this enables debug output for Orca.  The
+    # YYYY-MM-DD-HH:MM:SS portion is a shorthand way of saying that
+    # the file name will be formed from the current date and time with
+    # 'debug' in front and '.out' at the end.  The 'debug' and '.out'
+    # portions of this string should not be translated (i.e., it will
+    # always start with 'debug' and end with '.out', regardless of the
+    # locale.).
+    #
+    print "--debug                      " + \
+          _("Send debug output to debug-YYYY-MM-DD-HH:MM:SS.out")
+
+    # Translators: this enables debug output for Orca and overrides
+    # the name of the debug file Orca will use for debug output if the
+    # --debug option is used.
+    #
+    print "--debug-file=filename        " + \
+          _("Send debug output to the specified file")
+
     # Translators: this is the description of the command line option
     # '-s, --setup, --gui-setup' that will initially display a GUI dialog
     # that would allow the user to set their Orca preferences.
@@ -1426,11 +1539,16 @@ def usage():
     print "-q, --quit                   " + \
           _("Quits Orca (if shell script used)")
 
-    print
+    # Translators: this is the Orca command line option to tell Orca to
+    # replace any existing Orca process(es) that might be running.
+    #
+    print "--replace                    " +\
+          _("Replace a currently running Orca")
 
     # Translators: this is text being sent to a terminal and we want to
     # keep the text lines within terminal boundaries.
     #
+    print
     print _("If Orca has not been previously set up by the user, Orca\n" \
             "will automatically launch the preferences set up unless\n" \
             "the -n or --no-setup option is used.")
@@ -1453,6 +1571,9 @@ def main():
     an exit code of 0 means normal completion and an exit code of 50
     means Orca exited because of a hang."""
 
+    global _debugSwitch
+    global _debugFile
+
     # Method to call when we think something might be hung.
     #
     settings.timeoutCallback = timeout
@@ -1505,6 +1626,7 @@ def main():
         # n is for no setup
         # t is for text setup
         # v is for version
+        # z is for no exit
         #
         opts, args = getopt.getopt(
             arglist,
@@ -1518,7 +1640,10 @@ def main():
              "text-setup",
              "no-setup",
              "list-apps",
-             "version"])
+             "debug",
+             "debug-file=",
+             "version",
+             "replace"])
         for opt, val in opts:
             if opt in ("-u", "--user-prefs-dir"):
                 userPrefsDir = val.strip()
@@ -1575,6 +1700,13 @@ def main():
             if opt in ("-v", "--version"):
                 print "Orca %s" % platform.version
                 die(0)
+            if opt == "--debug":
+                _debugSwitch = True
+                if not _debugFile:
+                    _debugFile = time.strftime('debug-%Y-%m-%d-%H:%M:%S.out')
+            if opt == "--debug-file":
+                _debugSwitch = True
+                _debugFile = val.strip()
             if opt in ("-l", "--list-apps"):
                 apps = filter(lambda x: x is not None,
                               pyatspi.Registry.getDesktop(0))
