@@ -91,12 +91,14 @@ pound                =  u'\u00a3'
 yen                  =  u'\u00a5'
 section              =  u'\u00a7'
 copyright_sign       =  u'\u00a9'
+left_double_angle    =  u'\u00ab'
 not_sign             =  u'\u00ac'
 registered           =  u'\u00ae'
 degree               =  u'\u00b0'
 plus_minus           =  u'\u00b1'
 superscript2         =  u'\u00b2'
 superscript3         =  u'\u00b3'
+right_double_angle   =  u'\u00bb'
 one_quarter          =  u'\u00bc'
 one_half             =  u'\u00bd'
 three_quarters       =  u'\u00be'
@@ -116,6 +118,7 @@ prime                =  u'\u2032'
 double_prime         =  u'\u2033'
 euro                 =  u'\u20ac'
 trademark            =  u'\u2122'
+left_arrow           =  u'\u2190'
 right_arrow          =  u'\u2192'
 infinity             =  u'\u221e'
 almost_equal         =  u'\u2248'
@@ -198,7 +201,10 @@ punctuation[prime]              =  [ LEVEL_SOME, PUNCTUATION_REPLACE ]
 punctuation[double_prime]       =  [ LEVEL_SOME, PUNCTUATION_REPLACE ]
 punctuation[per_mille]          =  [ LEVEL_SOME, PUNCTUATION_REPLACE ]
 
+punctuation[left_arrow]         =  [ LEVEL_NONE, PUNCTUATION_REPLACE ]
 punctuation[right_arrow]        =  [ LEVEL_NONE, PUNCTUATION_REPLACE ]
+punctuation[left_double_angle]  =  [ LEVEL_NONE, PUNCTUATION_REPLACE ]
+punctuation[right_double_angle] =  [ LEVEL_NONE, PUNCTUATION_REPLACE ]
 punctuation[middle_dot]         =  [ LEVEL_NONE, PUNCTUATION_REPLACE ]
 punctuation[bullet]             =  [ LEVEL_NONE, PUNCTUATION_REPLACE ]
 punctuation[triangular_bullet]  =  [ LEVEL_NONE, PUNCTUATION_REPLACE ]
